@@ -3,8 +3,9 @@ package Catalyst::Plugin::PageCache;
 use strict;
 use base qw/Class::Accessor::Fast/;
 use MRO::Compat;
+use Digest::SHA1 ();
 
-our $VERSION = '0.22';
+our $VERSION = '0.30';
 
 # Do we need to cache the current page?
 __PACKAGE__->mk_accessors('_cache_page');
@@ -18,84 +19,115 @@ __PACKAGE__->mk_accessors('_page_cache_key');
 
 sub cache_page {
     my ( $c, @args ) = @_;
+    my %options;
 
-    # Parameters passed in?
+    # Mark that cache_page has been called but defer on selecting the time
+    # period to cache for until finalize. _get_page_cache_expiration_time has
+    # more options at that time, such as various headers which may have been set.
 
-    if ( ref($args[0]) eq 'HASH' || @args > 1 ) {
-        my $options = ref( $args[0] ) ? shift : { @args };
+    if (@args == 1) {
+        my $expires = shift @args;
 
-        $options->{cache_seconds} = $c->config->{'Plugin::PageCache'}->{expires}
-            unless exists $options->{cache_seconds};
+        # Allow specific end time
+        $expires = $expires->epoch - time()
+            if ref($expires) && $expires->can('epoch');
 
-        $c->_cache_page( $options );
-        return;
+        $options{cache_seconds} = $expires;
+    }
+    else {
+        %options = @args;
     }
 
-    my $expires = shift @args;
-
-    # Allow specific end time
-    $expires = $expires->epoch - time
-        if ref($expires) && $expires->isa('DateTime');
-
-    $expires ||= $c->config->{'Plugin::PageCache'}->{expires};
-
-    # mark the page for caching during finalize
-    if ( $expires > 0 ) {
-        $c->_cache_page( { cache_seconds => $expires } );
-    }
+    $c->_cache_page( \%options );
 }
 
+
 sub clear_cached_page {
     my ( $c, $uri ) = @_;
 
-    return unless ( $c->can( 'cache' ) );
+    return undef unless $c->can( 'cache' );
+
+    my $pc_config = $c->config->{'Plugin::PageCache'};
+    my $is_debug  = $pc_config->{debug};
     
     # Warn if index was disabled
-    if ( $c->config->{'Plugin::PageCache'}->{disable_index} ) {
-        $c->log->warn("Warning: clear_cached_page($uri) did not clear the cache, disable_index is set");
+    my $index_page_key = $pc_config->{index_page_key} or do {
+        $c->log->warn("clear_cached_page($uri) did not clear the cache because disable_index is set");
         return;
-    }
+    };
     
-    my $is_debug = $c->config->{'Plugin::PageCache'}->{debug};
+    my $cache = $c->cache; # curry the cache just once, here
+
+    my $index = $cache->get( $index_page_key ) || {};
 
     my $removed = 0;
-    
-    my $index = $c->cache->get( "_page_cache_index" ) || {};
+    foreach my $index_key ( keys %$index ) {
 
-    foreach my $key ( keys %{$index} ) {
-        if ( $key =~ /^(?::[^:]+:)?$uri$/xms ) {
-            $c->cache->remove( $key );
-            delete $index->{$key};
-            $removed++;
-        
-            $c->log->debug( "Removed $key from page cache" ) if $is_debug;
-        }
+        # find matching entries, ignoring the language prefix
+        next unless $index_key =~ /^ $uri (?:\#.*)? $/x;
+
+        # delete from the index and remove the corresponding cache entry
+        # (which may have a different key, e.g., sha1 encoded)
+        my $cache_key = delete $index->{$index_key};
+        $cache->remove( $cache_key );
+        $removed++;
+
+        $c->log->debug( "Removed $index_key from page cache" ) if $is_debug;
     }
 
-    if ( $removed ) {
-        $c->cache->set(
-            "_page_cache_index",
-            $index,
-            $c->config->{'Plugin::PageCache'}->{no_expire}
-        );
-    }
+    $cache->set( $index_page_key, $index, $pc_config->{no_expire} )
+        if $removed;
+
+    $removed;
 }
 
+
+# return the time that the item should expire
+sub _get_page_cache_expiration_time {
+    my ($c, $options) = @_;
+    my $pc_config = $c->config->{'Plugin::PageCache'};
+    my $is_debug = $pc_config->{debug};
+
+    my $expires;
+
+    # Use the explicitely passed in duration if available.
+    # XXX but why then ignore it if it's <= 0 ?
+    if ($options->{cache_seconds} and $options->{cache_seconds} > 0) {
+        $c->log->debug("expires in specified $options->{cache_seconds}s")
+            if $is_debug;
+
+        $expires = time() + $options->{cache_seconds};
+    }
+    # else {
+    #   ... calculate expiry based on the response headers
+    # }
+    # If all else fails, fallback to the default 'expires' configuration value.
+    else {
+        $c->log->debug("expires in default $pc_config->{expires}s")
+            if $is_debug;
+        $expires = time() + $pc_config->{expires};
+    }
+
+    return $expires;
+}
+
+
 sub dispatch {
     my $c = shift;
 
-    # never serve POST request pages from cache
-    return $c->next::method(@_) if ( $c->req->method eq "POST" );
+    # only serve GET and HEAD request pages from cache
+    # (never POST, PUT, DELETE etc)
+    return $c->next::method(@_)
+        unless $c->req->method =~ m/^(?:GET|HEAD)$/;
 
-    my $hook =
-        $c->config->{'Plugin::PageCache'}->{cache_hook}
-      ? $c->can($c->config->{'Plugin::PageCache'}->{cache_hook})
-      : undef;
-    
+    my $pc_config = $c->config->{'Plugin::PageCache'};
+
+    my $hook_name = $pc_config->{cache_dispatch_hook} || $pc_config->{cache_hook};
+    my $hook = $hook_name ? $c->can($hook_name) : undef;
     return $c->next::method(@_) if ( $hook && !$c->$hook() );
 
     return $c->next::method(@_)
-      if ( $c->config->{'Plugin::PageCache'}->{auto_check_user}
+      if ( $pc_config->{auto_check_user}
         && $c->can('user_exists')
         && $c->user_exists);
 
@@ -103,33 +135,35 @@ sub dispatch {
     return $c->next::method(@_)
         unless my $key = $c->_get_page_cache_key;
 
+    my $cache = $c->cache; # curry the cache just once, here
+
     return $c->next::method(@_)
-        unless my $data = $c->cache->get( $key );
+        unless my $data = $cache->get( $key );
 
     # Time to remove page from cache?
 
     if ( $data->{expire_time} && $data->{expire_time} <= time ) {
-        if ( my $busy_lock = $c->config->{'Plugin::PageCache'}->{busy_lock} ) {
+        if ( my $busy_lock = $pc_config->{busy_lock} ) {
             # Extend the expiration time for others while
             # this caller refreshes the cache
             $data->{expire_time} = time() + $busy_lock;
             
-            $c->cache->set( $key, $data );
+            $cache->set( $key, $data );
             
             $c->log->debug( "$key has expired, being refreshed for $busy_lock seconds" )
-                if ($c->config->{'Plugin::PageCache'}->{debug});
+                if ($pc_config->{debug});
         }
         else {
             $c->log->debug( "Expiring $key from page cache" )
-              if ($c->config->{'Plugin::PageCache'}->{debug});
+              if ($pc_config->{debug});
 
-            $c->cache->remove( $key );
+            $cache->remove( $key );
 
-            if ( !$c->config->{'Plugin::PageCache'}->{disable_index} ) {
-                my $index = $c->cache->get( "_page_cache_index" ) || {};
-                delete $index->{$key};
-                $c->cache->set( "_page_cache_index", $index,
-                    $c->config->{'Plugin::PageCache'}->{no_expire});
+            if ( my $index_page_key = $pc_config->{index_page_key} ) {
+                my $index = $cache->get( $index_page_key ) || {};
+                my $found = delete $index->{ $data->{index_key} };
+                $cache->set( $index_page_key, $index, $pc_config->{no_expire})
+                    if $found;
             }
         }
 
@@ -139,7 +173,7 @@ sub dispatch {
     $c->log->debug("Serving $key from page cache, expires in "
           . ($data->{expire_time} - time)
           . " seconds")
-      if ($c->config->{'Plugin::PageCache'}->{debug});
+        if ($pc_config->{debug});
 
     $c->_page_cache_used( 1 );
 
@@ -163,7 +197,8 @@ sub dispatch {
 }
 
 # See if request matches last_modified date in cache
-# and return true if so.
+# if so, arrange to return a 304 Not Modified status
+# and return true.
 
 sub _page_cache_not_modified {
     my ( $c, $data ) = @_;
@@ -186,41 +221,43 @@ sub _page_cache_not_modified {
 
 sub _set_page_cache_headers {
     my ( $c, $data ) = @_;
+    my $headers = $c->res->headers;
+    my $pc_config = $c->config->{'Plugin::PageCache'};
 
-    if ( $c->config->{'Plugin::PageCache'}->{cache_headers} ) {
+    if ( $pc_config->{cache_headers} ) {
         for my $header_key ( keys %{ $data->{headers} || {} } ) {
-            $c->res->headers->header(
+            $headers->header(
                 $header_key => $data->{headers}->{$header_key}
             );
         }
     }
     
-    return unless $c->config->{'Plugin::PageCache'}->{set_http_headers};
+    return unless $pc_config->{set_http_headers};
 
     if ( exists $data->{expires} ) {
 
         # page cache but not client cache
         if ( !$data->{expires} ) {
-            $c->res->headers->header( 'Cache-Control' => 'no-cache' );
-            $c->res->headers->header( 'Pragma' => 'no-cache' );
+            $headers->header( 'Cache-Control' => 'no-cache' );
+            $headers->header( 'Pragma' => 'no-cache' );
             return;
         }
 
-        $c->res->headers->header(
+        $headers->header(
             'Cache-Control' => "max-age=" . $data->{expires});
 
-        $c->res->headers->expires( time + $data->{expires} );
+        $headers->expires( time + $data->{expires} );
 
     }
     else {
 
-        $c->res->headers->header(
+        $headers->header(
             'Cache-Control' => "max-age=" . ($data->{expire_time} - time));
 
-        $c->res->headers->expires( $data->{expire_time} );
+        $headers->expires( $data->{expire_time} );
     }
 
-    $c->res->headers->last_modified( $data->{create_time} )
+    $headers->last_modified( $data->{create_time} )
         unless $c->res->status && $c->res->status == 304;
 }
 
@@ -230,14 +267,14 @@ sub finalize {
     # never cache POST requests
     return $c->next::method(@_) if ( $c->req->method eq "POST" );
 
-    my $hook =
-        $c->config->{'Plugin::PageCache'}->{cache_hook}
-      ? $c->can($c->config->{'Plugin::PageCache'}->{cache_hook})
-      : undef;
+    my $pc_config = $c->config->{'Plugin::PageCache'};
+
+    my $hook_name = $pc_config->{cache_dispatch_hook} || $pc_config->{cache_hook};
+    my $hook = $hook_name ? $c->can($hook_name) : undef;
     return $c->next::method(@_) if ( $hook && !$c->$hook() );
 
     return $c->next::method(@_)
-      if ( $c->config->{'Plugin::PageCache'}->{auto_check_user}
+      if ( $pc_config->{auto_check_user}
         && $c->can('user_exists')
         && $c->user_exists);
     return $c->next::method(@_) if ( scalar @{ $c->error } );
@@ -247,7 +284,7 @@ sub finalize {
     return $c->next::method(@_) if ( $c->_page_cache_used );
 
     if (!$c->_cache_page
-        && scalar @{ $c->config->{'Plugin::PageCache'}->{auto_cache} })
+        && scalar @{ $pc_config->{auto_cache} })
     {
 
         # is this page part of the auto_cache list?
@@ -255,70 +292,95 @@ sub finalize {
 
         # For performance, this should be moved to setup, and generate a hash.
         AUTO_CACHE:
-        foreach my $auto (@{ $c->config->{'Plugin::PageCache'}->{auto_cache} })
+        foreach my $auto (@{ $pc_config->{auto_cache} })
         {
             next if $auto =~ m/^\d$/;
             if ( $path =~ /^$auto$/ ) {
                 $c->log->debug( "Auto-caching page $path" )
-                    if ($c->config->{'Plugin::PageCache'}->{debug});
+                    if ($pc_config->{debug});
                 $c->cache_page;
                 last AUTO_CACHE;
             }
         }
     }
 
-    if ( my $options = $c->_cache_page ) {
-        my $key = $c->_get_page_cache_key;
-
-        my $now = time;
-
-        $c->log->debug(
-            "Caching page $key for $options->{cache_seconds} seconds"
-        ) if ($c->config->{'Plugin::PageCache'}->{debug});
-
-        # Cache some additional metadata along with the content
-        # Some caches don't support expirations, so we do it manually
-        my $data = {
-            body => $c->res->body || undef,
-            content_type => [ $c->res->content_type ] || undef,
-            content_encoding => $c->res->content_encoding || undef,
-            create_time      => $options->{last_modified}
-                || $c->res->headers->last_modified
-                || $now,
-            expire_time => $now + $options->{cache_seconds},
-        };
-        
-        if ( $c->config->{'Plugin::PageCache'}->{cache_headers} ) {
-            $data->{headers} = {
-                map { $_ => $c->res->headers->header($_) }
-                  $c->res->headers->header_field_names
-            };
-        }
-
-        $data->{expires} = $options->{expires} if exists $options->{expires};
-
-        $c->cache->set( $key, $data );
-
-        $c->_set_page_cache_headers( $data );  # don't forget the first time
-
-        if ( !$c->config->{'Plugin::PageCache'}->{disable_index} ) {
-            # Keep an index cache of all pages that have been cached, for use
-            # with clear_cached_page
-            my $index = $c->cache->get( "_page_cache_index" ) || {};
-            $index->{$key} = 1;
-
-            # Save date in cache
-            $c->cache->set( "_page_cache_index", $index,
-                $c->config->{'Plugin::PageCache'}->{no_expire});
-        }
+    if ($c->_cache_page) {
+        my $data = $c->_store_page_in_cache($c->_cache_page) ;
 
         # Check for If-Modified-Since
-        $c->_page_cache_not_modified( $data );
+        $c->_page_cache_not_modified( $data ) if $data;
     }
 
     return $c->next::method(@_);
 }
 
+
+sub _store_page_in_cache {
+    my ($c, $options) = @_;
+
+    my $key = $c->_get_page_cache_key;
+    my $pc_config = $c->config->{'Plugin::PageCache'};
+    my $now = time();
+
+    my $headers = $c->res->headers;
+
+    # Cache some additional metadata along with the content
+    # Some caches don't support expirations, so we do it manually
+    my $data = {
+        body => $c->res->body || undef,
+        content_type => [ $c->res->content_type ] || undef,
+        content_encoding => $c->res->content_encoding || undef,
+        create_time      => $options->{last_modified}
+            || $headers->last_modified
+            || $now,
+        expire_time => $c->_get_page_cache_expiration_time($options),
+    };
+
+    return undef if $data->{expire_time} <= $now;
+
+    $c->log->debug(
+        "Caching page $key for ". ($data->{expire_time} - time()) ." seconds"
+    ) if ($pc_config->{debug});
+    
+    if ( $pc_config->{cache_headers} ) {
+        $data->{headers} = {
+            map { $_ => $headers->header($_) } $headers->header_field_names
+        };
+    }
+
+    if ($pc_config->{index_page_key}) {
+        # We can't simply use $key for $index_key because $key may have been
+        # mangled by sha1 and/or a key_maker hook.  We include $key to ensure
+        # unique entries in the index in cases where a given uri might produce
+        # different results eg due to headers like language.
+        $data->{index_key}  = '/'.$c->req->path;
+        $data->{index_key} .= '?'.$c->req->uri->query if $c->req->uri->query;
+        $data->{index_key} .= '#'.$key;
+    }
+
+    if (exists $options->{expires}) {
+        $data->{expires} = $options->{expires}
+    }
+
+    my $cache = $c->cache; # curry the cache just once, here
+
+    $cache->set( $key, $data );
+
+    $c->_set_page_cache_headers( $data );  # don't forget the first time
+
+    if ( $data->{index_key} ) {
+        # Keep an index cache of all pages that have been cached, for use
+        # with clear_cached_page. This is error prone. See KNOWN ISSUES.
+        my $index_page_key = $pc_config->{index_page_key};
+        my $index = $cache->get($index_page_key) || {};
+        $index->{ $data->{index_key} } = $key;
+        $cache->set($index_page_key, $index, $pc_config->{no_expire});
+    }
+
+    return $data;
+}
+
+
 sub setup {
     my $c = shift;
 
@@ -329,34 +391,49 @@ sub setup {
         $c->config->{'Plugin::PageCache'} = delete $c->config->{page_cache};
     }
 
-    $c->config->{'Plugin::PageCache'}->{auto_cache}       ||= [];
-    $c->config->{'Plugin::PageCache'}->{expires}          ||= 60 * 5;
-    $c->config->{'Plugin::PageCache'}->{cache_headers}    ||= 0;
-    $c->config->{'Plugin::PageCache'}->{set_http_headers} ||= 0;
-    $c->config->{'Plugin::PageCache'}->{disable_index}    ||= 0;
-    $c->config->{'Plugin::PageCache'}->{busy_lock}        ||= 0;
-    $c->config->{'Plugin::PageCache'}->{debug}            ||= $c->debug;
+    my $pc_config = $c->config->{'Plugin::PageCache'} ||= {};
+
+    $pc_config->{auto_cache}       ||= [];
+    $pc_config->{expires}          ||= 60 * 5;
+    $pc_config->{cache_headers}    ||= 0;
+    $pc_config->{set_http_headers} ||= 0;
+    $pc_config->{busy_lock}        ||= 0;
+    $pc_config->{debug}            ||= $c->debug;
+
+    # default the page key to include the app name to give some measure
+    # of protection if the cache doesn't have a namespace set.
+    $pc_config->{index_page_key} = "$c._page_cache_index"
+        unless defined $pc_config->{index_page_key};
+
+    if (not defined $pc_config->{disable_index}) {
+        warn "Plugin::PageCache config does not include disable_index, which currently defaults false but may default true in future\n";
+        $pc_config->{disable_index} = 0;
+    }
+    $pc_config->{index_page_key} = undef
+        if $pc_config->{disable_index};
 
     # detect the cache plugin being used and set appropriate
     # never-expires syntax
     if ( $c->can('cache') ) {
-        
+
         # Newer C::P::Cache, cannot call $c->cache as a package method
         if ( $c->isa('Catalyst::Plugin::Cache') ) {
             return;
         }
 
+        my $cache = $c->cache; # curry the cache just once, here
+
         # Older Cache plugins
-        if ( $c->cache->isa('Cache::FileCache') ) {
-            $c->config->{'Plugin::PageCache'}->{no_expire} = "never";
+        if ( $cache->isa('Cache::FileCache') ) {
+            $pc_config->{no_expire} = "never";
         }
-        elsif ($c->cache->isa('Cache::Memcached')
-            || $c->cache->isa('Cache::FastMmap'))
+        elsif ($cache->isa('Cache::Memcached')
+            || $cache->isa('Cache::FastMmap'))
         {
 
           # Memcached defaults to 'never' when not given an expiration
           # In FastMmap, it's not possible to set an expiration
-            $c->config->{'Plugin::PageCache'}->{no_expire} = undef;
+            $pc_config->{no_expire} = undef;
         }
     }
     else {
@@ -367,11 +444,12 @@ sub setup {
 sub _get_page_cache_key {
     my $c = shift;
     
-    # We can't rely on the params after the user's code has run, so
-    # use the key created during the initial dispatch phase
+    # We can't rely on the params after the user's code has run,
+    # so we cache the key created during the initial dispatch phase
+    # and reuse it at finalize time.
     return $c->_page_cache_key if ( $c->_page_cache_key );
 
-    # override key if required
+    # override key if required, else use uri path
     my $keymaker = $c->config->{'Plugin::PageCache'}->{key_maker};
     my $key = $keymaker ? $keymaker->($c) : "/" . $c->req->path;
     
@@ -380,22 +458,31 @@ sub _get_page_cache_key {
         $key = ':' . $c->language . ':' . $key;
     }
 
-    if ( scalar $c->req->param ) {
-        my @params;
-        foreach my $arg ( sort keys %{ $c->req->params } ) {
-            if ( ref $c->req->params->{$arg} ) {
-                my $list = $c->req->params->{$arg};
-                push @params, map { "$arg=" . $_  } sort @{$list};
-            }
-            else {
-                push @params, "$arg=" . $c->req->params->{$arg};
-            }
-        }
-        $key .= '?' . join( '&', @params );
+    # some caches have limits on the max key length (eg for memcached it's 250
+    # minus the namespace length) so if the key is a non-trvial length then
+    # replace the tail with a sha1. (We only replace the tail because it's
+    # useful to be able to see the leading portion of the path in the debug logs)
+    if (length($key) > 100) {
+        substr($key, 100) = Digest::SHA1::sha1_hex($key); # 40 bytes
+    }
+    # Check for spaces as it's cheap insurance, just in case, for memcached
+    $key =~ s/\s/~/g;
+
+    # we always sha1 the parameters/query as they're typically long
+    # and it also avoids issues like spaces in keys for memcached
+    my $params_key;
+    my $parameters = $c->req->parameters;
+    if (%$parameters) {
+        local $Storable::canonical = 1;
+        $params_key = Storable::nfreeze($parameters);
     }
     elsif ( my $query = $c->req->uri->query ) {
-        $key .= '?' . $query;
+        $params_key = $query;
     }
+    # use sha1 to avoid problems with over-long params or params
+    # containing values that can't be used as a key (eg spaces for memcached)
+    $key .= '?' . Digest::SHA1::sha1_hex($params_key) # 40 bytes
+        if $params_key;
 
     $c->_page_cache_key( $key );
 
@@ -423,10 +510,19 @@ Catalyst::Plugin::PageCache - Cache the output of entire pages
                 '/list',
             ],
             debug => 1,
-            # Optionally, a cache hook to be called prior to dispatch to
+
+            # Optionally, a cache hook method to be called prior to dispatch to
             # determine if the page should be cached.  This is called both
             # before dispatch, and before finalize.
-            cache_hook => 'some_method'
+            cache_hook => 'some_method',
+
+            # You may alternatively set different methods to be used as hooks
+            # for dispatch and finalize. The dispatch method will determine
+            # whether the currently cached page will be displayed to the user,
+            # and the finalize hook will determine whether to save the newly
+            # created page.
+            cache_dispatch_hook => 'some_method_for_dispatch',
+            cache_finalize_hook => 'some_method_for_finalize',
         }
     );
     
@@ -443,7 +539,6 @@ Catalyst::Plugin::PageCache - Cache the output of entire pages
 
     $c->clear_cached_page( '/list' );
 
-
     # Expire at a specific time
     $c->cache_page( $datetime_object );
 
@@ -517,10 +612,20 @@ time.  URIs may be specified as absolute: '/list' or as a regex: '/view/.*'
 
     disable_index => 1
 
-In order to support the C<clear_cached_page> method, PageCache keeps an index of
-all cached pages.  If you don't intend to use C<clear_cached_page>, you may 
-enable this config option to avoid the overhead of creating and updating the
-cache index.  This option is disabled by default.
+To support the L</clear_cached_page> method, PageCache attempts keep an index
+of all cached pages. This adds overhead by performing extra cache reads and
+writes to maintain the (possibly very large) page index. It's also not
+reliable, see L</KNOWN ISSUES>.
+
+If you don't intend to use C<clear_cached_page>, you should enable this config
+option to avoid the overhead of creating and updating the cache index.  This
+option is currently disabled (i.e. the page index is enabled) by default but
+that may change in a future release.
+
+    index_page_key => '...'
+
+The key string used for the index, Defaults to a string that includes the name
+of the Catalyst app class.
 
     busy_lock => 10
 
@@ -537,11 +642,16 @@ previously cached page.  This option is disabled by default.
     debug => 1
 
 This will print additional debugging information to the Catalyst log.  You will
-need to have -Debug enabled to see these messages.  You can also specify an
-optional config parameter auto_check_user. If this option is enabled,
-automatic caching is disabled for logged in users.
+need to have -Debug enabled to see these messages.
+
+    auto_check_user => 1
+
+If this option is enabled, automatic caching is disabled for logged in users
+i.e., if the app class has a user_exists() method and it returns true.
 
     cache_hook => 'cache_hook_method'
+    cache_finalize_hook => 'cache_finalize_hook_method'
+    cache_dispatch_hook => 'cache_dispatch_hook_method'
 
 Calls a method on the application that is expected to return a true or false.
 This method is called before dispatch, and before finalize so you can short
@@ -575,6 +685,19 @@ To override the generation of page keys:
         }
     );
 
+C<key_maker> can also be the name of a method, which will be invoked as C<<$c->$key_maker>>.
+
+In most cases you would use a single cache_hook method for consistency.
+
+It is possible to achieve background refreshing of content by disabling
+caching in cache_dispatch_hook and enabling caching in cache_finalize_hook
+for a specific IP address (say 127.0.0.1).
+
+A cron of wget "http://localhost/foo.html" would cause the content to be
+generated fresh and cached for future viewers. Useful for content which 
+takes a very long time to build or pages which should be refreshed at
+a specific time such as always rolling over content at midnight.
+
 =head1 METHODS
 
 =head2 cache_page
@@ -586,7 +709,7 @@ Call cache_page in any controller method you wish to be cached.
 The page will be cached for $expire seconds.  Every user who visits the URI(s)
 referenced by that controller will receive the page directly from cache.  Your
 controller will not be processed again until the cache expires.  You can set
-this value to as low as 60 seconds if you have heavy traffic to greatly
+this value to a low value, such as 60 seconds, if you have heavy traffic, to greatly
 improve site performance.
 
 Pass in a DateTime object to make the cache expire at a given point in time.
@@ -614,19 +737,15 @@ Possible options are:
 
 =over 4
 
-=item last_modified
-
-Last modified time in epoch seconds.  If not set will use either the
-current Last-Modified header, or if not set, the current time.
-
 =item cache_seconds
 
 This is the number of seconds to keep the page in the page cache, which may be
 different (normally longer) than the time that client caches may store the page.
+This is the value set when only a single parameter is passed.
 
 =item expires
 
-This is the lenght of time in seconds that a client may cache the page
+This is the length of time in seconds that a client may cache the page
 before revalidating (by asking the server if the document has changed).
 
 Unlike above, this is a fixed setting that each client will see.  Regardless of
@@ -637,20 +756,31 @@ Setting zero (0) for expires will result in the page being cached, but headers
 will be sent telling the client to not cache the page.  Allows caching expensive
 content to generate, but any changes will be seen right away.
 
+=item last_modified
+
+Last modified time in epoch seconds.  If not set will use either the
+current Last-Modified header, or if not set, the current time.
+
 =back
 
 =head2 clear_cached_page
 
-To clear the cached value for a URI, you may call clear_cached_page.
+To clear the cached pages for a URI, you may call clear_cached_page.
 
     $c->clear_cached_page( '/view/userlist' );
     $c->clear_cached_page( '/view/.*' );
+    $c->clear_cached_page( '/view/.*\?.*\bparam=value\b' );
 
-This method takes an absolute path or regular expression.  For obvious reasons,
-this must be called from a different controller than the cached controller. You
-may for example wish to build an admin page that lets you clear page caches.
+This method takes an absolute path or regular expression.
+Returns the number of matching entries found in the index.
+A warning will be generated if the page index is disabled (see L</CONFIGURATION>).
 
-Note that clear_cached_page will generate a warning if disable_index is enabled.
+The argument is matched against all the keys in the page index.
+The page index keys include the request path and query string, if any.
+
+Typically you'd call this from a different controller than the cached
+controller. You may for example wish to build an admin page that lets you clear
+page caches.
 
 =head1 INTERNAL EXTENDED METHODS
 
@@ -673,6 +803,10 @@ separate cache key will be used for each language a page is displayed in.
 
 =head1 KNOWN ISSUES
 
+The page index, used to support the L</clear_cached_page> method is unreliable
+because it uses a read-modify-write approach which will loose data if more than
+one process attempts to update the page index at the same time.
+
 It is not currently possible to cache pages served from the Static plugin.  If
 you're concerned enough about performance to use this plugin, you should be
 serving static files directly from your web server anyway.
